/*
============================================================================
 Name        : DarkSideTetrisContainer.h
 Author      : 
 Copyright   : Your copyright notice
 Description : Declares container control for application.
============================================================================
*/

#ifndef DARKSIDETETRISCONTAINER_H
#define DARKSIDETETRISCONTAINER_H

// INCLUDES
#include <coecntrl.h>

// FORWARD DECLARATIONS
class CEikLabel;        // for example labels
class CGameState;
class CFbsBitmap;
class CPlayerAdapter;

// CLASS DECLARATION

/**
*  CDarkSideTetrisContainer  container control class.
*
*/
class CDarkSideTetrisContainer : public CCoeControl, MCoeControlObserver
    {
    public: // Constructors and destructor

        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CDarkSideTetrisContainer();

    public: // New functions
    
    	CGameState* GetGameState () const;
    	void SetGameState (CGameState* aGameState);
		void StartGameTimers (TInt32 interval);
		static TInt UpdateGame (TAny*);
    
    public: // Functions from base classes

    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;
        void DrawTitleScreen(const TRect& aRect) const;
        void DrawGame(const TRect& aRect) const;        
        void DrawScores(const TRect& aRect) const;        
		void DrawGameTable(const TRect& aRect, TInt16 aTableX, TInt16 aTableY, TBool aLarge) const;
		void DrawGameBlock(const TRect& aRect, TInt8 aBlockCode, TInt16 aX, TInt16 aY, TBool aLarge) const;

		void UpdateTimers ();

		/**
		* From MCoeControlObserver
		* Acts upon changes in the hosted control's state.
		*
		* @param	aControl	The control changing its state
		* @param	aEventType	The type of control event
		*/
        void HandleControlEventL(CCoeControl* aControl,TCoeEvent aEventType);

    private: //data
        CPeriodic* iGameTimer;
    	CGameState* iGameState;
        CFbsBitmap* iTitleLogo;
		CFbsBitmap* iTableBackground;
		CFbsBitmap* iBlockImage;
		CFbsBitmap* iGameOver;
    };

#endif


