#ifndef GAMETABLE_H_
#define GAMETABLE_H_

class CGameTable {

private:
    CStone* iCurrentStone;

    TInt8 block[DS_GAMETABLE_WIDTH][DS_GAMETABLE_HEIGHT];
    TInt8 iLevel;
    TInt16 iScore;

public:
	static CGameTable* NewL();
	static CGameTable* NewLC();
	
    virtual ~CGameTable();
    bool AddStone(CStone* aStone);
    void AddToScore(TInt16 aPoints);
    TInt8 CheckForRows();
    void Clear();
    TInt8 GetBlock(TInt8 aX, TInt8 aY);
    CStoneGenerator* GetStoneGenerator() const;
    void NextFrame(TTime aTime);
    void NextLevel();
    void Reset();
    TBool SetBlock(TInt8 aBlock, TInt8 aX, TInt8 aY, TBool aReleaseOld = true);
    CStone* GetCurrentStone() const;
    TBool SetCurrentStone(CStone* aCurrentStone);
    TInt8 GetLevel() const;
    void SetLevel(TInt8 aLevel);
    TInt16 GetScore() const;
    void SetScore(TInt16 aScore);
	void CheckRows ();


	TTime iNextLevelUpdateTime;
	
	TInt32 iTableUpdateInterval;
	TBool iTableUpdateIntervalChanged;

private:

	/**
        * Constructor for performing 1st stage construction
        */
	CGameTable();

	/**
        * EPOC default constructor for performing 2nd stage construction
        */
	void ConstructL();

};

#endif // GAMETABLE_H_

#ifndef _GAMETABLE_H_INLINES
#define _GAMETABLE_H_INLINES

inline TInt8 CGameTable::GetBlock(TInt8 aX, TInt8 aY)
{
  if (aX >= DS_GAMETABLE_WIDTH || aY >= DS_GAMETABLE_HEIGHT || aX < 0 || aY < 0)
    return 0;
  return block[aX][aY];
}

inline TBool CGameTable::SetBlock(TInt8 aBlock, TInt8 aX, TInt8 aY,
                                  TBool aReleaseOld)
{
  TBool overwrite = false;
  
  if (aX >= DS_GAMETABLE_WIDTH || aY >= DS_GAMETABLE_HEIGHT || aX < 0 || aY < 0)
    return true;

  if (block[aX][aY]) {
    overwrite = true;
    if (aReleaseOld) {
      block[aX][aY] = 0;
    }
  }
  
  block[aX][aY] = aBlock;
  return overwrite;
}

inline CStone* CGameTable::GetCurrentStone() const
{
  return iCurrentStone;
}

inline TInt8 CGameTable::GetLevel() const
{
  return iLevel;
}

inline void CGameTable::SetLevel(TInt8 aLevel)
{
  if (aLevel > 20)
    aLevel = 20;
  
  iLevel = aLevel;
}

inline TInt16 CGameTable::GetScore() const
{
  return iScore;
}

inline void CGameTable::SetScore(TInt16 aScore)
{
  if (aScore < 0)
    iScore = 0;
  else
    iScore = aScore;
}

#endif // ndef _GAMETABLE_H_INLINES
