/*
============================================================================
 Name        : CDarkSideTetrisAppUi.cpp
 Author      : 
 Copyright   : Your copyright notice
 Description : CDarkSideTetrisAppUi implementation
============================================================================
*/

// INCLUDE FILES
#include "DarkSideTetrisAppui.h"
#include "DarkSideTetrisContainer.h"
#include <DarkSideTetris.rsg>
#include "DarkSideTetris.hrh"

#include <avkon.hrh>
#include <aknnotewrappers.h>
#include <e32keys.h>

#include "DarkSideTetris.h"

// ================= MEMBER FUNCTIONS =======================
//
// ----------------------------------------------------------
// CDarkSideTetrisAppUi::ConstructL()
//
// ----------------------------------------------------------
//
void CDarkSideTetrisAppUi::ConstructL()
    {
    BaseConstructL();

    iAppContainer = new (ELeave) CDarkSideTetrisContainer;
    iAppContainer->SetMopParent( this );
    iAppContainer->ConstructL( ClientRect() );
    AddToStackL( iAppContainer );

	// Create the game's state
	iGameState = CGameState::NewL ();
	CStone* newStone = iGameState->iStoneGenerator->BuildRandom();
	iGameState->iGameTable->SetCurrentStone(newStone);
    iAppContainer->SetGameState( iGameState );
    
    _LIT(KHighScoreSound, "c:\\system\\Apps\\DarkSideTetris\\dshighscores.wav");
    
    iStartGameSound = CDarkSidePlayerAdapter::NewL (KHighScoreSound);

    }

// ----------------------------------------------------
// CDarkSideTetrisAppUi::~CDarkSideTetrisAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CDarkSideTetrisAppUi::~CDarkSideTetrisAppUi()
    {
    if (iAppContainer)
        {
        RemoveFromStack( iAppContainer );
        delete iAppContainer;
        }
        
   	delete iGameState;
    delete iStartGameSound;
   }

// ------------------------------------------------------------------------------
// CDarkSideTetrisAppUi::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//
void CDarkSideTetrisAppUi::DynInitMenuPaneL(
    TInt /*aResourceId*/,CEikMenuPane* /*aMenuPane*/)
    {
    }

// ----------------------------------------------------
// CDarkSideTetrisAppUi::HandleKeyEventL(
//     const TKeyEvent& aKeyEvent,TEventCode /*aType*/)
// takes care of key event handling
// ----------------------------------------------------
//
TKeyResponse CDarkSideTetrisAppUi::HandleKeyEventL(
    const TKeyEvent& aKeyEvent,TEventCode aType)
    {
	
	switch(aType) {
		
		case EEventKeyUp:
//		case EEventKeyDown:
			// If we are playing
			if (iGameState->iGlobalState == DS_GAMESTATE_PLAYING) {
				CStone* currentStone = iGameState->iGameTable->GetCurrentStone();
				
				if (currentStone) {
					if(aKeyEvent.iScanCode == EStdKeyNkp4 || aKeyEvent.iScanCode == EStdKeyLeftArrow) {
          				currentStone->MoveLeft (false);
          				break;
					} else 
					if(aKeyEvent.iScanCode == EStdKeyNkp6 || aKeyEvent.iScanCode == EStdKeyRightArrow) {
						currentStone->MoveRight (false);
          				break;
					} else
					if(aKeyEvent.iScanCode == EStdKeyNkp7) {
						currentStone->DropLeft ();
          				break;
					} else
					if(aKeyEvent.iScanCode == EStdKeyNkp9) {
						currentStone->DropRight ();
          				break;
					} else
					if(aKeyEvent.iScanCode == EStdKeyNkp2 || aKeyEvent.iScanCode == EStdKeyDownArrow) {
        				currentStone->MoveDown (false);
          				break;
					} else
					if(aKeyEvent.iScanCode == EStdKeyNkp3 || aKeyEvent.iScanCode == EStdKeyNkp8) {
          				currentStone->RotateRight (false);
          				break;
					} else
					if(aKeyEvent.iScanCode == EStdKeyNkp1 || aKeyEvent.iScanCode == EStdKeyUpArrow) {
          				currentStone->RotateLeft (false);
          				break;
					} else
          			if(aKeyEvent.iScanCode == EStdKeyNkp5 || aKeyEvent.iScanCode == EStdKeyDevice3) {
        				currentStone->Drop ();
						break;
          			} else {
          				return EKeyWasNotConsumed;
          			}
					return EKeyWasConsumed;
										
				}		         
			} else {
				return EKeyWasConsumed;
			}
			
			break;

		case EEventKey:
//		case EEventKeyUp:
		case EEventKeyDown:
		
		

			return EKeyWasConsumed;
			break;

	}
	
	iAppContainer->DrawNow ();
	return EKeyWasConsumed;
	
    }

// ----------------------------------------------------
// CDarkSideTetrisAppUi::HandleCommandL(TInt aCommand)
// takes care of command handling
// ----------------------------------------------------
//
void CDarkSideTetrisAppUi::HandleCommandL(TInt aCommand)
    {
    switch ( aCommand )
        {
        case EAknSoftkeyBack:
        case EEikCmdExit:
            {
			CAknQueryDialog* dlg = CAknQueryDialog::NewL();
			dlg->SetPromptL( _L("Do You really want to exit?") );
			TInt result = dlg->ExecuteLD( R_DARKSIDETETRIS_EXIT_QUERY );
			if(result)
			{
            	Exit();
			}


            break;
            }
        case EDarkSideTetrisCmdAppNewGame:
            {
            TBuf<6> iPlayerName;
			while (iPlayerName.Length () == 0) {
				CAknTextQueryDialog* dlg = CAknTextQueryDialog::NewL(iPlayerName, CAknQueryDialog::EWarningTone);
				dlg->SetPromptL( _L("Enter player name:") );
				dlg->SetMaxLength(5);
				dlg->ExecuteLD( R_DARKSIDETETRIS_PLAYERNAME_QUERY );
			}
			
			iGameState->iGameTable->Reset ();
			iGameState->iNextStoneCode = iGameState->iStoneGenerator->BuildRandomCode ();;

            iGameState->iPlayerName = iPlayerName;			
            	
           	iGameState->iGlobalState = DS_GAMESTATE_PLAYING;
            	
            iAppContainer->StartGameTimers (iGameState->iGameTable->iTableUpdateInterval);
            break;
            }

        case EDarkSideTetrisCmdAppHighScores:
			{
			if (iGameState->iGlobalState == DS_GAMESTATE_PLAYING) {
				CAknQueryDialog* dlg = CAknQueryDialog::NewL();
				dlg->SetPromptL( _L("Do You really want to stop the game?") );
				TInt result = dlg->ExecuteLD( R_DARKSIDETETRIS_EXIT_QUERY );
				delete dlg;
				
				if(result) {
					iGameState->iGlobalState = DS_GAMESTATE_HIGHSCORES;
					iGameState->iGameTable->Reset ();
					iStartGameSound->PlayL();
					iGameState->iScoreBoard->LoadL();
				}
			} else {
				iGameState->iGlobalState = DS_GAMESTATE_HIGHSCORES;
				iStartGameSound->PlayL();
				iGameState->iScoreBoard->LoadL();
			}
			iAppContainer->DrawNow ();
			
			}

        default:
            break;
        }
    }




