/*
============================================================================
 Name        : DarkSideTetrisContainer.cpp
 Author      : 
 Copyright   : Your copyright notice
 Description : Container control implementation
============================================================================
*/

// INCLUDE FILES
#include "DarkSideTetrisContainer.h"
#include "DarkSideTetris.h"
#include "FBS.h"

#include <eiklabel.h>  // for example label control
#include <eikenv.h>

#include <DarkSideTetris.mbg>
#include <aknutils.h>
#include <MdaAudioSamplePlayer.h>

_LIT (KBitmapFilename, "DarkSideTetris.mbm");

#define DRAWBLOCKNEXTSTONE(c, x, y) \
        DrawGameBlock (aRect, c,\
          (tableX + tableWidth + 40) + ((x - 5) * DS_BLOCK_SIZE), \
          (tableY + 180) + (y * DS_BLOCK_SIZE), \
          true);


// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CDarkSideTetrisContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CDarkSideTetrisContainer::ConstructL(const TRect& aRect)
    {
    CreateWindowL();

  	iGameTimer = CPeriodic::NewL(CActive::EPriorityIdle);
  	
  	TFileName fullName (KBitmapFilename);
  	CompleteWithAppPath (fullName);
  	
  	// Load the bitmaps
	iTitleLogo = new (ELeave) CFbsBitmap ();
	User::LeaveIfError(iTitleLogo->Load (fullName, EMbmDarksidetetrisTetrislogo));

	iBlockImage = new (ELeave) CFbsBitmap ();
	User::LeaveIfError(iBlockImage->Load (fullName, EMbmDarksidetetrisBlocks));

	iTableBackground = new (ELeave) CFbsBitmap ();
	User::LeaveIfError(iTableBackground->Load (fullName, EMbmDarksidetetrisTablebackground));

	iGameOver = new (ELeave) CFbsBitmap ();
	User::LeaveIfError(iGameOver->Load (fullName, EMbmDarksidetetrisEnd));
	
    SetRect(aRect);
    ActivateL();
    }

// Destructor
CDarkSideTetrisContainer::~CDarkSideTetrisContainer()
    {
    delete iGameTimer;
    delete iTitleLogo;
	delete iTableBackground;
	delete iBlockImage;
	delete iGameOver;
    }

// ---------------------------------------------------------
// CDarkSideTetrisContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CDarkSideTetrisContainer::SizeChanged()
    {
    }

// ---------------------------------------------------------
// CDarkSideTetrisContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CDarkSideTetrisContainer::CountComponentControls() const
    {
    return 0; // return nbr of controls inside this container
    }

// ---------------------------------------------------------
// CDarkSideTetrisContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CDarkSideTetrisContainer::ComponentControl(TInt aIndex) const
    {
    switch ( aIndex )
        {
        default:
            return NULL;
        }
    }

void CDarkSideTetrisContainer::DrawGameTable(const TRect& aRect, TInt16 aTableX, TInt16 aTableY, TBool aLarge) const {
	CWindowGc& gc = SystemGc();

	gc.SetPenStyle( CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
	gc.SetBrushColor( KRgbBlack );

/*
    TRect tableBack;
	tableBack.SetRect(aTableX, aTableY, aTableX + DS_BLOCK_SIZE * DS_GAMETABLE_WIDTH, aTableY + DS_BLOCK_SIZE * (DS_GAMETABLE_HEIGHT - DS_FIRST_VISIBLE_ROW));
	gc.DrawRect(tableBack);
*/

	// HACK:
	aTableY += 15;

    TSize bgsize = iTableBackground->SizeInPixels();
    TPoint bgpoint (aTableX - 1, aTableY - 1);
    gc.BitBlt(bgpoint, iTableBackground);


	for (TInt8 i = 0; i < DS_GAMETABLE_WIDTH; i++) {
		for (TInt8 j = DS_FIRST_VISIBLE_ROW; j < DS_GAMETABLE_HEIGHT; j++) {
	    	TInt8 bl = iGameState->iGameTable->GetBlock(i, j);
	    	if (bl != 0) {
	      		DrawGameBlock (aRect, bl, 
	        		aTableX + (i * DS_BLOCK_SIZE), 
	        		aTableY + ((j - DS_FIRST_VISIBLE_ROW) * DS_BLOCK_SIZE),
	        		aLarge);
			}
		}
	}
	
	if (iGameState->iGlobalState == DS_GAMESTATE_GAMEOVER) {
	    TSize bgsize = iGameOver->SizeInPixels();
    	TPoint gopoint (aTableX + 10, aTableY + 120);
	    gc.BitBlt(gopoint, iGameOver);		
	}
}
		
void CDarkSideTetrisContainer::DrawGameBlock(const TRect& aRect, TInt8 aBlockCode, TInt16 aX, TInt16 aY, TBool aLarge) const {
	CWindowGc& gc = SystemGc();
	
	gc.SetPenStyle( CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
/*  
	TRgb blockColor;
	if (aBlockCode == DS_BLOCK_RED)
		blockColor = TRgb (255, 0, 0);
	else if (aBlockCode == DS_BLOCK_BLUE)
		blockColor = TRgb (0, 0, 255);
	else if (aBlockCode == DS_BLOCK_CYAN)
		blockColor = TRgb (128, 128, 255);
	else if (aBlockCode == DS_BLOCK_MAGENTA)
		blockColor = TRgb (255, 0, 255);
	else if (aBlockCode == DS_BLOCK_YELLOW)
		blockColor = TRgb (255, 255, 0);
	else if (aBlockCode == DS_BLOCK_ORANGE)
		blockColor = TRgb (255, 128, 64);
	else if (aBlockCode == DS_BLOCK_GREEN)
		blockColor = TRgb (0, 255, 0);
	else if (aBlockCode == DS_BLOCK_WHITE)
		blockColor = TRgb (255, 255, 255);
	else if (aBlockCode == DS_BLOCK_GREY)
		blockColor = TRgb (128, 128, 128);
		
	gc.SetBrushColor( blockColor );
	

	TRect drawBlock;
	drawBlock.SetRect(aX, aY, aX + DS_BLOCK_SIZE, aY + DS_BLOCK_SIZE);
    gc.DrawRect(drawBlock);
*/
    
    TRect blockSource (((aBlockCode - 1) * DS_BLOCK_SIZE), 0, ((aBlockCode - 1) * DS_BLOCK_SIZE) + 9, 9);
    TPoint blockPoint (aX,aY);
    gc.BitBlt(blockPoint, iBlockImage, blockSource);
}

void CDarkSideTetrisContainer::DrawGame(const TRect& aRect) const {
    CWindowGc& gc = SystemGc();	

	TInt16 maxX = Rect ().Width();
	TInt16 maxY = Rect ().Height();
    
	gc.SetPenStyle( CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
	gc.SetBrushColor( TRgb (0, 0, 160) );    
	
	TRect gameBack;
	gameBack.SetRect(0, 0, maxX, maxY);
    gc.DrawRect(gameBack);

	TInt16 tableWidth = (DS_GAMETABLE_WIDTH * DS_BLOCK_SIZE);
	TInt16 tableHeight = (DS_GAMETABLE_HEIGHT - DS_FIRST_VISIBLE_ROW) * DS_BLOCK_SIZE;

	// 70 is for the texts
	TInt16 tableX = (maxX - tableWidth - 70) / 2;
	TInt16 tableY = (maxY - tableHeight) / 2;    
	DrawGameTable(aRect, tableX, tableY, true);    

 	// Draw player name
	const CFont* iPlayerFont = iEikonEnv->LegendFont();
	gc.UseFont(iPlayerFont);
	gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
	gc.SetBrushColor( TRgb (0, 0, 160) );    

	TInt margin=0;

	TRect box(tableX + tableWidth + 10, tableY, tableX + tableWidth + 60, tableY + 20);
	TInt baseline = box.Height() /2 + iPlayerFont->AscentInPixels()/2; 

	box = TRect(tableX + tableWidth + 10, tableY + 20, tableX + tableWidth + 70, tableY + 40);
	gc.SetPenColor(TRgb (128, 128, 128));
	_LIT(KTxtPlayer,"Name");
	gc.DrawText(KTxtPlayer, box, baseline, CGraphicsContext::ELeft, margin);
	
	box = TRect(tableX + tableWidth + 10, tableY + 45, tableX + tableWidth + 70, tableY + 65);
	gc.SetPenColor(TRgb (255, 255, 255));
	gc.DrawText(iGameState->iPlayerName	, box, baseline, CGraphicsContext::ERight, margin);

	// Draw points
	box = TRect(tableX + tableWidth + 10, tableY + 65, tableX + tableWidth + 70, tableY + 85);
	gc.SetPenColor(TRgb (128, 128, 128));
	_LIT(KTxtScore,"Score");
	gc.DrawText(KTxtScore, box, baseline, CGraphicsContext::ELeft, margin);

	TBuf<6> scoreStr;
	scoreStr.AppendNum(iGameState->iGameTable->GetScore());
	box = TRect(tableX + tableWidth + 10, tableY + 90, tableX + tableWidth + 70, tableY + 110);
	gc.SetPenColor(TRgb (255, 255, 255));
	gc.DrawText(scoreStr, box, baseline, CGraphicsContext::ERight, margin);

	// Draw level
	box = TRect(tableX + tableWidth + 10, tableY + 110, tableX + tableWidth + 70, tableY + 130);
	_LIT(KTxtLevel,"Level");
	gc.SetPenColor(TRgb (128, 128, 128));
	gc.DrawText(KTxtLevel, box, baseline, CGraphicsContext::ELeft, margin);

	TBuf<6> levelStr;
	levelStr.AppendNum(iGameState->iGameTable->GetLevel ());
	box = TRect(tableX + tableWidth + 10, tableY + 130, tableX + tableWidth + 70, tableY + 150);
	gc.SetPenColor(TRgb (255, 255, 255));
	gc.DrawText(levelStr, box, baseline, CGraphicsContext::ERight, margin);

	// Draw next stone
	// Draw level
	box = TRect(tableX + tableWidth + 10, tableY + 155, tableX + tableWidth + 70, tableY + 175);
	_LIT(KTxtNext,"Next");
	gc.SetPenColor(TRgb (128, 128, 128));
	gc.DrawText(KTxtNext, box, baseline, CGraphicsContext::ELeft, margin);

	
     switch (iGameState->iNextStoneCode) {
        case DS_STONE_ORANGE_SQUARE:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_ORANGE, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_ORANGE, 5, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_ORANGE, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_ORANGE, 5, 2);
          break;

        case DS_STONE_RED_STICK:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_RED, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_RED, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_RED, 4, 1);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_RED, 4, 0);
          break;

        case DS_STONE_BLUE_CORNER:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_BLUE, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_BLUE, 5, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_BLUE, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_BLUE, 4, 1);
          break;

        case DS_STONE_GREEN_CORNER:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_GREEN, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_GREEN, 5, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_GREEN, 5, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_GREEN, 5, 1);
          break;

        case DS_STONE_PINK_CURVE:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_MAGENTA, 5, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_MAGENTA, 5, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_MAGENTA, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_MAGENTA, 4, 1);
          break;

        case DS_STONE_YELLOW_CURVE:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_YELLOW, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_YELLOW, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_YELLOW, 5, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_YELLOW, 5, 1);
          break;

        case DS_STONE_CYAN_CROSSING:
          DRAWBLOCKNEXTSTONE (DS_BLOCK_CYAN, 4, 3);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_CYAN, 4, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_CYAN, 5, 2);
          DRAWBLOCKNEXTSTONE (DS_BLOCK_CYAN, 4, 1);
          break;

        default:
          break;
      }	

}

void CDarkSideTetrisContainer::DrawTitleScreen(const TRect& aRect) const {
    CWindowGc& gc = SystemGc();	

	TInt16 maxX = Rect ().Width();
	TInt16 maxY = Rect ().Height();
    
	gc.SetPenStyle( CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
	gc.SetBrushColor( TRgb (0, 0, 0) );    
	
	TRect titleBack;
	titleBack.SetRect(0, 0, maxX, maxY);
    gc.DrawRect(titleBack);	

	gc.SetBrushColor( TRgb (0, 0, 160) );    

    TSize logoSize = iTitleLogo->SizeInPixels();
    TPoint logo ((maxX - logoSize.iWidth) / 2, (maxX - logoSize.iHeight) / 2);
    gc.BitBlt(logo, iTitleLogo);
       
}

void CDarkSideTetrisContainer::DrawScores(const TRect& aRect) const {
    CWindowGc& gc = SystemGc();	

	TInt16 maxX = Rect ().Width();
	TInt16 maxY = Rect ().Height();
    
	gc.SetPenStyle( CGraphicsContext::ESolidPen);
    gc.SetBrushStyle( CGraphicsContext::ESolidBrush );
	gc.SetBrushColor( TRgb (0, 0, 0) );    
	
	TRect titleBack;
	titleBack.SetRect(0, 0, maxX, maxY);
    gc.DrawRect(titleBack);	

	// Draw scores

	TInt margin=0;

	const CFont* iTitleFont = iEikonEnv->TitleFont();
	gc.UseFont(iTitleFont);


	gc.SetPenColor(TRgb (255, 255, 255));


	TInt16 titleWidth = 200;
	TInt16 titleHeight = 30;

	TRect box((maxX - titleWidth) / 2, 20 / 2, (maxX + titleWidth) / 2, 20 + titleHeight);
	TInt baseline = box.Height() /2 + iTitleFont->AscentInPixels()/2; 

	_LIT(KTxtTitle,"HIGH SCORES");
	gc.DrawText(KTxtTitle, box, baseline, CGraphicsContext::ECenter, margin);	

	const CFont* iScoreFont = iEikonEnv->LegendFont();
	gc.UseFont(iScoreFont);

	TInt16 nameWidth = 70;
	TInt16 nameHeight = 20;
	TInt16 scoreWidth = 70;
	TInt16 scoreHeight = 20;
	TInt16 totalWidth = nameWidth + scoreWidth + 10;

	TInt16 nameX = (maxX - totalWidth) / 2;
	TInt16 nameY = 20 + titleHeight + 10;
	TInt16 scoreX = nameX + nameWidth + 10;



	box = TRect(nameX, nameY, nameX + nameWidth, nameY + nameHeight);
	baseline = box.Height() /2 + iScoreFont->AscentInPixels()/2; 
	
	// Draw first players 
	box = TRect (nameX, nameY, nameX + nameWidth, nameY + nameHeight);
	baseline = box.Height() /2 + iScoreFont->AscentInPixels()/2; 

	_LIT(KTxtName,"Name");
	gc.DrawText(KTxtName, box, baseline, CGraphicsContext::ELeft, margin);	

	box = TRect (scoreX, nameY, scoreX + scoreWidth, nameY + scoreHeight);
	_LIT(KTxtScore,"Score");
	gc.DrawText(KTxtScore, box, baseline, CGraphicsContext::ERight, margin);	

	for (TInt8 i = 0; i < iGameState->iScoreBoard->GetScoreCount(); i++) {
		nameY += 30;
	
		CScore* s = iGameState->iScoreBoard->GetScore (i);
		box = TRect(nameX, nameY, nameX + nameWidth, nameY + nameHeight);
		baseline = box.Height() /2 + iScoreFont->AscentInPixels()/2; 
		gc.DrawText(s->iName, box, baseline, CGraphicsContext::ELeft, margin);
	
		TBuf<6> scoreStr;
		scoreStr.AppendNum(s->iPoints);
		box = TRect(scoreX, nameY, scoreX + scoreWidth, nameY + scoreHeight);
		gc.DrawText(scoreStr, box, baseline, CGraphicsContext::ERight, margin);
		
	}


}

// ---------------------------------------------------------
// CDarkSideTetrisContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CDarkSideTetrisContainer::Draw(const TRect& aRect) const
    {
    CWindowGc& gc = SystemGc();

	if (iGameState->iGlobalState == DS_GAMESTATE_TITLESCREEN) {
		DrawTitleScreen(aRect);
	} else
	if (iGameState->iGlobalState == DS_GAMESTATE_PLAYING || iGameState->iGlobalState == DS_GAMESTATE_GAMEOVER) {
		DrawGame(aRect);
	}	
	if (iGameState->iGlobalState == DS_GAMESTATE_HIGHSCORES) {
		DrawScores(aRect);
	}	

    }

// ---------------------------------------------------------
// CDarkSideTetrisContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CDarkSideTetrisContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
    {
    // TODO: Add your control event handler code here
    }

CGameState* CDarkSideTetrisContainer::GetGameState () const {
	return iGameState;
}

void CDarkSideTetrisContainer::SetGameState (CGameState* aGameState) {
	if (iGameState)
		delete iGameState;
	iGameState = aGameState;
}

void CDarkSideTetrisContainer::StartGameTimers (TInt32 interval) {
	delete iGameTimer;
	iGameTimer = CPeriodic::NewL(CActive::EPriorityIdle);
	iGameTimer->Start(interval, interval, TCallBack (UpdateGame, this));
	iGameState->iStartGameTime.HomeTime ();
	iGameState->iGameTable->iNextLevelUpdateTime = iGameState->iStartGameTime;
	TTimeIntervalMicroSeconds sec;
	sec = DS_LEVEL_UPDATE_INTERVAL;
	iGameState->iGameTable->iNextLevelUpdateTime += sec;
}

TInt CDarkSideTetrisContainer::UpdateGame (TAny* aPtr) { 
	((CDarkSideTetrisContainer*)aPtr)->UpdateTimers ();
	return 0;
}

void CDarkSideTetrisContainer::UpdateTimers () {
	iGameState->NextFrame();
	if (iGameState->iGameTable->iTableUpdateIntervalChanged) {
		StartGameTimers (iGameState->iGameTable->iTableUpdateInterval);
		iGameState->iGameTable->iTableUpdateIntervalChanged = EFalse;
	}
	DrawNow ();
}
