#include "DarkSideTetris.h"

CGameTable::CGameTable()
{
}

CGameTable::~CGameTable()
{
   delete iCurrentStone;
}


CGameTable* CGameTable::NewLC()
{
	CGameTable* self = new (ELeave)CGameTable();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
}

CGameTable* CGameTable::NewL()
{
	CGameTable* self=CGameTable::NewLC();
	CleanupStack::Pop(); // self;
	return self;
}

void CGameTable::ConstructL()
{
	iCurrentStone = 0;
  
	TInt i, j;
	for (i = 0; i < DS_GAMETABLE_WIDTH; i++) {
		for (j = 0; j < DS_GAMETABLE_HEIGHT; j++) {
			block[i][j] = 0;
		}
	}

	iScore = 0;	
	iLevel = 1;
	iTableUpdateInterval = 500000;
	iTableUpdateIntervalChanged = false;
	iNextLevelUpdateTime.HomeTime();

}

bool CGameTable::AddStone(CStone* aStone)
{
  switch (aStone->GetCode ()) {
    case DS_STONE_ORANGE_SQUARE:
      SetBlock (DS_BLOCK_ORANGE, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_ORANGE, 5, 3);
      aStone->AddBlock (5, 3);
      SetBlock (DS_BLOCK_ORANGE, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_ORANGE, 5, 2);
      aStone->AddBlock (5, 2);
      break;

    case DS_STONE_RED_STICK:
      SetBlock (DS_BLOCK_RED, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_RED, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_RED, 4, 1);
      aStone->AddBlock (4, 1);
      SetBlock (DS_BLOCK_RED, 4, 0);
      aStone->AddBlock (4, 0);
      break;

    case DS_STONE_BLUE_CORNER:
      SetBlock (DS_BLOCK_BLUE, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_BLUE, 5, 3);
      aStone->AddBlock (5, 3);
      SetBlock (DS_BLOCK_BLUE, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_BLUE, 4, 1);
      aStone->AddBlock (4, 1);
      break;

    case DS_STONE_GREEN_CORNER:
      SetBlock (DS_BLOCK_GREEN, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_GREEN, 5, 3);
      aStone->AddBlock (5, 3);
      SetBlock (DS_BLOCK_GREEN, 5, 2);
      aStone->AddBlock (5, 2);
      SetBlock (DS_BLOCK_GREEN, 5, 1);
      aStone->AddBlock (5, 1);
      break;

    case DS_STONE_PINK_CURVE:
      SetBlock (DS_BLOCK_MAGENTA, 5, 3);
      aStone->AddBlock (5, 3);
      SetBlock (DS_BLOCK_MAGENTA, 5, 2);
      aStone->AddBlock (5, 2);
      SetBlock (DS_BLOCK_MAGENTA, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_MAGENTA, 4, 1);
      aStone->AddBlock (4, 1);
      break;

    case DS_STONE_YELLOW_CURVE:
      SetBlock (DS_BLOCK_YELLOW, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_YELLOW, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_YELLOW, 5, 2);
      aStone->AddBlock (5, 2);
      SetBlock (DS_BLOCK_YELLOW, 5, 1);
      aStone->AddBlock (5, 1);
      break;

    case DS_STONE_CYAN_CROSSING:
      SetBlock (DS_BLOCK_CYAN, 4, 3);
      aStone->AddBlock (4, 3);
      SetBlock (DS_BLOCK_CYAN, 4, 2);
      aStone->AddBlock (4, 2);
      SetBlock (DS_BLOCK_CYAN, 5, 2);
      aStone->AddBlock (5, 2);
      SetBlock (DS_BLOCK_CYAN, 4, 1);
      aStone->AddBlock (4, 1);
      break;

    default:
      return 0;
  }

  aStone->SetGameTable (this);

  if (!aStone->MoveDown (true)) {
    return false;
  }

  return true;
}


void CGameTable::AddToScore(TInt16 aPoints)
{
  iScore += aPoints;
}

TInt8 CGameTable::CheckForRows()
{
  TInt i, j = 0;
  TInt rowscleared = 0;
  TBool rowfull = true;

  while (rowfull) {
    for (i = DS_GAMETABLE_HEIGHT - 1; i > 1; i--) {
      rowfull = true;
      for (j = 0; j < DS_GAMETABLE_WIDTH; j++) {
        if (block[j][i] == 0) {
          rowfull = false;
          break;
        }
      }
      if (rowfull) {
        rowscleared++;
      }
    }
  }
  return rowscleared;
}

void CGameTable::Clear()
{
  TInt i, j;

  for (i = 0; i < DS_GAMETABLE_WIDTH; i++) {
    for (j = 0; j < DS_GAMETABLE_HEIGHT; j++) {
      if (iCurrentStone) {
        if (!iCurrentStone->Contains (i,j))
          SetBlock (0, i, j);
      } else {
        SetBlock (0, i, j);
      }
    }
  }
}

void CGameTable::NextFrame(TTime aCurrentTime)
{

}

void CGameTable::NextLevel()
{
	if (GetLevel () < 20)
		SetLevel (GetLevel () + 1);
    
	iTableUpdateInterval = (500 - ((GetLevel ()- 1) * 25)) * 1000;
	iTableUpdateIntervalChanged = ETrue;    

}

TBool CGameTable::SetCurrentStone(CStone* aCurrentStone)
{
	delete iCurrentStone;
  iCurrentStone = aCurrentStone;
  if (iCurrentStone)
    return AddStone (iCurrentStone);
  else
    return true;
}


void CGameTable::CheckRows()
{
 	TInt8 j = 0;
	TBool rowfull = true;
	TInt8 rowscleared = 0;
	TInt8 block = 0;
	TInt8 bcode = 0;

	while (rowfull) {
		for (TInt8 i = DS_GAMETABLE_HEIGHT - 1; i > 1; i--) {
			rowfull = true;
			for (j = 0; j < DS_GAMETABLE_WIDTH; j++) {
				if (GetBlock(j, i) == 0) {
					rowfull = false;
					break;
				}
			}
			if (rowfull) {
/*				for (int l = 0; l < DS_GAMETABLE_WIDTH; l++) {
					ClearBlock (aGameTable, GetBlock (l,i));
				}*/

//				SetRowChanged (i - 1);

				rowscleared++;
				for (int k = i - 1; k > 0; k--) {
					for (int l = 0; l < DS_GAMETABLE_WIDTH; l++) {
						block = GetBlock (l,k);
						if (block) 
							bcode = block;
						else
							bcode = 0;
						
						SetBlock (0, l, k);
						SetBlock (bcode, l,k + 1, false);

					}
				}

				break;
			}
		}
	}

	if (rowscleared == 1) {
		AddToScore (DS_POINTS_CLEAR_ROW);
	} else if (rowscleared == 2) {
		AddToScore (DS_POINTS_CLEAR_2ROWS);
	} else if (rowscleared == 3) {
		AddToScore (DS_POINTS_CLEAR_3ROWS);
	} else if (rowscleared == 4) {
		AddToScore (DS_POINTS_CLEAR_4ROWS);
	}
}

void CGameTable::Reset () {
  TInt i, j;

	SetCurrentStone (0);

	for (i = 0; i < DS_GAMETABLE_WIDTH; i++) {
		for (j = 0; j < DS_GAMETABLE_HEIGHT; j++) {
			SetBlock (0, i, j);
		}
	}


	SetScore (0);
	SetLevel (1);
}