/*
============================================================================
 Name        : ScoreBoard.cpp
 Author      : 
 Version     :
 Copyright   : Your copyright notice
 Description : CScoreBoard implementation
============================================================================
*/

#include <f32file.h>
#include <s32file.h>
#include <s32std.h>
#include "DarkSideTetris.h"

CScoreBoard::CScoreBoard()
{
	// No implementation required
}


CScoreBoard::~CScoreBoard()
{
	iScores.ResetAndDestroy();
}

CScoreBoard* CScoreBoard::NewLC()
{
	CScoreBoard* self = new (ELeave)CScoreBoard();
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
}

CScoreBoard* CScoreBoard::NewL()
{
	CScoreBoard* self=CScoreBoard::NewLC();
	CleanupStack::Pop(); // self;
	return self;
}

void CScoreBoard::ConstructL()
{
}

void CScoreBoard::LoadL () {
	iScores.ResetAndDestroy();

	RFs fileSession;
	User::LeaveIfError (fileSession.Connect ());
	CleanupClosePushL(fileSession);
	
	_LIT(KFileName, "dsscores.dat");
	RFileReadStream inStream;
	TInt error = inStream.Open(fileSession, KFileName, EFileRead);
	if (error != KErrNone) {
		// Failure
		CleanupStack::PopAndDestroy ();
		return;
	}
	CleanupClosePushL(inStream);	


	TInt scoreCount = inStream.ReadInt16L();
	for (TInt i = 0; i < scoreCount; i++) {
		CScore* score = CScore::NewL();
		score->InternalizeL(inStream);
		AddScore (score);
	}
	
	CleanupStack::PopAndDestroy ();
	CleanupStack::PopAndDestroy ();
}

void CScoreBoard::SaveL () {
	RFs fileSession;
	User::LeaveIfError (fileSession.Connect ());
	CleanupClosePushL(fileSession);
	
	_LIT(KFileName, "dsscores.dat");
	RFileWriteStream outStream;
	TInt error = outStream.Replace (fileSession, KFileName, EFileWrite);
	if (error != KErrNone) {
		// Failure
		CleanupStack::PopAndDestroy ();
		return;
	}
	CleanupClosePushL(outStream);

	outStream.WriteInt16L(iScores.Count ());
	for (TInt i = 0; i < iScores.Count(); i++) {
		iScores[i]->ExternalizeL(outStream);
	}
	
	CleanupStack::PopAndDestroy();
	CleanupStack::PopAndDestroy();
}


void CScoreBoard::AddScore (CScore* aNewScore) {
	TBool didInsert = EFalse;
	for (TInt i = 0; i < iScores.Count(); i++) {
		if (aNewScore->iPoints > iScores[i]->iPoints) {
			iScores.Insert (aNewScore, i);			
			didInsert = ETrue;
			break;
		}
	}
	if (!didInsert) {
		iScores.Append(aNewScore);
	}
	// Keep max 5 scores
	while (iScores.Count () > 5) {
		delete iScores[5];
		iScores.Remove(5);
	}
}
	
CScore* CScoreBoard::GetScore (TInt8 aIndex) {
	return iScores[aIndex];
}

TInt8 CScoreBoard::GetScoreCount () {
	return iScores.Count ();
}

/*
Directory list

_LIT (KDir, "C:\\");
CDir* dirList;
User::LeaveIfError (iFileServer.GetDir (KFir, KEntryAttMaskSupported, ESortByName, dirList));
for (TInt i = 0; i < dirList->Count (); i++) {
	TPtrC fileName = (*dirList)[i].iName;
}
*/
