#include "DarkSideTetris.h"

CStone::CStone()
  : blockCount(0)
  , code(0)
  , uglyCheck(false)
{
	iGameTable = 0;
}

CStone::CStone(TInt8 aStoneCode)
  : blockCount(0)
  , code(aStoneCode)
  , uglyCheck(false)
{
	iGameTable = 0;
}

CStone::~CStone()
{

}

void CStone::AddBlock(TInt8 aX, TInt8 aY)
{
  if (blockCount == DS_BLOCKSINSTONE_COUNT)
    return;

  blockX[blockCount] = aX;
  blockY[blockCount] = aY;
  blockCount++;
}

TBool CStone::Contains(TInt8 aX, TInt8 aY)
{
  for (TInt8 i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (blockX[i] == aX && blockY[i] == aY)
      return true;
  }
  return false;
}

void CStone::Drop()
{
	TBool hasSpace = true;
	while (hasSpace) {
		hasSpace = MoveDown (false);
	}
  	GetGameTable ()->SetCurrentStone (0);
}

void CStone::DropLeft()
{
	TBool hasSpace = true;
	while (hasSpace) {
		hasSpace = MoveLeft (false);
	}
}

void CStone::DropRight()
{
	TBool hasSpace = true;
	while (hasSpace) {
		hasSpace = MoveRight (false);
	}
}

void CStone::Empty()
{
	blockCount = 0;
}

TInt8 CStone::GetBlock(TInt8 aIndex)
{
  return iGameTable->GetBlock (blockX[aIndex], blockY[aIndex]);
}

TInt8 CStone::GetBlockX(TInt8 aIndex)
{
  return blockX[aIndex];
}

TInt8 CStone::GetBlockY(TInt8 aIndex)
{
  return blockY[aIndex];
}

TBool CStone::MoveDown(TBool aTest)
{
  TInt8 bl = 0;
  TInt8 oldX[4];
  TInt8 oldY[4];
  TInt8 newX[4];
  TInt8 newY[4];
  TInt8 block[4];
  TInt8 i = 0;

  // Load coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    oldX[i] = blockX[i];
    oldY[i] = blockY[i];
    block[i] = GetGameTable ()->GetBlock (blockX[i], blockY[i]);
  }

  // Calculate new coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    newX[i] = oldX[i];
    newY[i] = oldY[i] + 1;
  }

  // Test new coordinates
  TBool free = true;
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (newY[i] >= DS_GAMETABLE_HEIGHT) {
	  free = false;
      break;
    }
    bl = GetGameTable ()->GetBlock (newX[i], newY[i]);
    if (bl) {
      if (!Contains (newX[i], newY[i])){
        free = false;
        break;
      }
    }
  }

  // If we were testing, return
  if (aTest)
    return free;
    
  // Wrap it up
  if (free) {
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      blockX[i] = newX[i];
      blockY[i] = newY[i];
      GetGameTable ()->SetBlock (0, oldX[i], oldY[i]);
    }
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      GetGameTable ()->SetBlock (block[i], newX[i], newY[i]);
    }
    GetGameTable ()->AddToScore (DS_POINTS_MOVE_DOWN);  
    return true;
  }
  return false;
}


TBool CStone::MoveLeft(TBool aTest)
{
  TInt8 bl = 0;
  TInt8 oldX[4];
  TInt8 oldY[4];
  TInt8 newX[4];
  TInt8 newY[4];
  TInt8 block[4];
  TInt8 i = 0;

  // Load coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    oldX[i] = blockX[i];
    oldY[i] = blockY[i];
    block[i] = GetGameTable ()->GetBlock (oldX[i], oldY[i]);
  }

  // Calculate new coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    newX[i] = oldX[i] - 1;
    newY[i] = oldY[i];
  }

  // Test new coordinates
  TBool free = true;
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (newX[i] < 0) {
	  free = false;
      break;
	}
    bl = GetGameTable ()->GetBlock (newX[i], newY[i]);
    if (bl) {
      if (!Contains (newX[i], newY[i])) {
        free = false;
        break;
      }
    }
  }

  // If we were testing, return
  if (aTest)
    return free;
    
  // Wrap it up
  if (free) {
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      blockX[i] = newX[i];
      blockY[i] = newY[i];
      GetGameTable ()->SetBlock (0, oldX[i], oldY[i]);
    }
    for (i = 0; i < GetBlockCount (); i++) {
      GetGameTable ()->SetBlock (block[i], newX[i], newY[i]);
    }
    return true;
  }
  return false;
}


TBool CStone::MoveRight(TBool aTest)
{
  TInt8 bl = 0;
  TInt8 oldX[4];
  TInt8 oldY[4];
  TInt8 newX[4];
  TInt8 newY[4];
  TInt8 block[4];
  TInt8 i = 0;

  // Load coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    oldX[i] = blockX[i];
    oldY[i] = blockY[i];
    block[i] = GetGameTable ()->GetBlock (blockX[i], blockY[i]);
  }

  // Calculate new coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    newX[i] = oldX[i] + 1;
    newY[i] = oldY[i];
  }

  // Test new coordinates
  TBool free = true;
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (newX[i] >= DS_GAMETABLE_WIDTH) {
	  free = false;
      break;
	}
    bl = GetGameTable ()->GetBlock (newX[i], newY[i]);
    if (bl) {
      if (!Contains (newX[i], newY[i])) {
        free = false;
        break;
      }
    }
  }

  // If we were testing, return
  if (aTest)
    return free;
    
  // Wrap it up
  if (free) {
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      blockX[i] = newX[i];
      blockY[i] = newY[i];
      GetGameTable ()->SetBlock (0, oldX[i], oldY[i]);
    }
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      GetGameTable ()->SetBlock (block[i], newX[i], newY[i]);
    }
    GetGameTable ()->AddToScore (DS_POINTS_MOVE_DOWN);  
    return true;
  }
  return false;
}

TBool CStone::RotateLeft(TBool aTest)
{//@CODE_1028
  TInt8 oldX[4];
  TInt8 oldY[4];
  TInt8 newX[4];
  TInt8 newY[4];
  TInt8 block[4];
  TInt8 leftx = DS_GAMETABLE_WIDTH;
  TInt8 rightx = 0;
  TInt8 upy = DS_GAMETABLE_HEIGHT; 
  TInt8 downy = 0;
  TInt8 bl = 0;
  TInt8 i;

  // Find the borders of the stone
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    TInt8 x = blockX[i];
    TInt8 y = blockY[i];
    block[i] = GetGameTable ()->GetBlock (x, y);
		if (x > rightx)
		  rightx = x;
		if (x < leftx)
			leftx = x;
		if (y > downy)
			downy = y;
		if (y < upy)
			upy = y;
  }
  TInt8 middlex = (leftx + rightx) / 2;
  TInt8 middley = (upy + downy) / 2;

  // Load coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
		oldX[i] = blockX[i];
		oldY[i] = blockY[i];
  }

	// Calculate new coordinates
	for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    TInt8 x = oldX[i];
		TInt8 y = oldY[i];
		if (x <= middlex) {
			if (y <= middley) {
				newX[i] = middlex - middley + y;
				newY[i] = middlex - x + middley + 1;
			} else {
				newX[i] = middlex + y - middley;
				newY[i] = middlex - x + middley + 1;
			}
		} else {
			if (y <= middley) {
				newX[i] = middlex - middley + y;
				newY[i] = middley - x + middlex + 1;
			} else {
				newX[i] = y - middley + middlex;
				newY[i] = middley - x + middlex + 1;
			}
		}
	}

  // Test new coordinates
  TBool free = true;
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (newX[i] < 0 || newX[i] > DS_GAMETABLE_WIDTH - 1 || newY[i] > DS_GAMETABLE_HEIGHT - 1) {
		  free = false;
      break;
	  }
    bl = GetGameTable ()->GetBlock (newX[i], newY[i]);
    if (bl) {
      if (!Contains (newX[i], newY[i])) {
        free = false;
        break;
      }
    }
  }

  // If we were testing, return
  if (aTest)
    return free;
    
  // Wrap it up
  if (free) {
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      blockX[i] = newX[i];
      blockY[i] = newY[i];
      GetGameTable ()->SetBlock (0, oldX[i], oldY[i]);
    }
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      GetGameTable ()->SetBlock (block[i], newX[i], newY[i]);
    }
    return true;
  }
  return false;
}


TBool CStone::RotateRight(TBool aTest)
{
  TInt8 oldX[4];
  TInt8 oldY[4];
  TInt8 newX[4];
  TInt8 newY[4];
  TInt8 block[4];
  TInt8 leftx = DS_GAMETABLE_WIDTH;
  TInt8 rightx = 0;
  TInt8 upy = DS_GAMETABLE_HEIGHT;
  TInt8 downy = 0;
  TInt8 bl = 0;
  TInt8 i;

  // Find the borders of the stone
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    TInt8 x = blockX[i];
		TInt8 y = blockY[i];
    block[i] = GetGameTable ()->GetBlock (x, y);
		if (x > rightx)
		  rightx = x;
		if (x < leftx)
			leftx = x;
		if (y > downy)
			downy = y;
		if (y < upy)
			upy = y;
  }
  TInt8 middlex = (leftx + rightx) / 2;
	TInt8 middley = (upy + downy) / 2;

  // Load coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
		oldX[i] = blockX[i];
		oldY[i] = blockY[i];
  }

	// Calculate new coordinates
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
		TInt8 x = oldX[i];
		TInt8 y = oldY[i];
		if (x <= middlex) {
			if (y <= middley) {
				newX[i] = middley - y + middlex + 1;
				newY[i] = middley - middlex + x;
			} else {
				newX[i] = middley - y + middlex + 1;
				newY[i] = middley - middlex + x;
			}
		} else {
			if (y <= middley) {
				newX[i] = middley - y + middlex + 1;
				newY[i] = middley - middlex + x;
			} else {
				newX[i] = y - middley + middlex - 1;
				newY[i] = middley - middlex + x;
			}
		}
	}

	if (uglyCheck != false) {
		for (TInt8 i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
			newX[i]--;
		}
		uglyCheck = false;
	} else {
		uglyCheck = true;
	}

  // Test new coordinates
  bool free = true;
  for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
    if (newX[i] < 0 || newX[i] > DS_GAMETABLE_WIDTH - 1 || newY[i] > DS_GAMETABLE_HEIGHT - 1) {
		  free = false;
      break;
	  }
    bl = GetGameTable ()->GetBlock (newX[i], newY[i]);
    if (bl) {
      if (!Contains (newX[i], newY[i])) {
        free = false;
        break;
      }
    }
  }

  // If we were testing, return
  if (aTest)
    return free;
    
  // Wrap it up
  if (free) {
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      blockX[i] = newX[i];
      blockY[i] = newY[i];
      GetGameTable ()->SetBlock (0, oldX[i], oldY[i]);
    }
    for (i = 0; i < DS_BLOCKSINSTONE_COUNT; i++) {
      GetGameTable ()->SetBlock (block[i], newX[i], newY[i]);
    }
    return true;
  }
  return false;
}
